package openio

import (
	"context"
	"fmt"
	"time"

	"github.com/open-io/oio-exporter/common"
	"github.com/open-io/oio-exporter/daemon"
)

var SCORE_SERVICE_TYPES = map[string]struct{}{
	"account":    struct{}{},
	"beanstalkd": struct{}{},
	"meta0":      struct{}{},
	"meta1":      struct{}{},
	"meta2":      struct{}{},
	"oioproxy":   struct{}{},
	"oioswift":   struct{}{},
	"rawx":       struct{}{},
	"rdir":       struct{}{},
}

type openioScoreDaemonParams struct {
	Namespace   string
	ServiceType string
	// Those need to be sorted for comparison
	ScoreServices []*scoreService
	ProxyAddrs    []string
}

type openioScoreDaemon struct {
	daemon.BaseDaemon

	namespace     string
	serviceType   string
	scoreServices []*scoreService
	proxyAddrs    []string

	proxyI int
}

type scoreService struct {
	ID     string
	IP     string
	Port   int
	Volume string
}

type serviceInfoBody struct {
	Addr  string
	Score int
}

func (d *openioScoreDaemon) Interval() time.Duration {
	return confInterval
}

func (d *openioScoreDaemon) Collect(ctx context.Context, m *common.MetricData) error {
	serviceType := d.serviceType
	proxyAddrs := d.proxyAddrs
	d.proxyI = (d.proxyI + 1) % len(proxyAddrs)
	proxyAddr := proxyAddrs[d.proxyI]

	url := fmt.Sprintf("http://%s/v3.0/%s/conscience/list?type=%s", proxyAddr, d.namespace, serviceType)
	requestCtx, cancel := context.WithTimeout(ctx, confTimeout)
	defer cancel()

	info := []serviceInfoBody{}
	err := common.HTTPGetJson(requestCtx, url, &info)
	if err != nil {
		m.IncrError("openio", serviceType)
		return err
	}

	for _, scoreService := range d.scoreServices {
		serviceAddr := fmt.Sprintf("%s:%d", scoreService.IP, scoreService.Port)

		for _, serviceInfo := range info {
			if serviceInfo.Addr != serviceAddr {
				continue
			}

			m.Set(&common.Metric{
				Name: "score",
				Labels: [][]string{
					[]string{"namespace", d.namespace},
					[]string{"service", scoreService.ID},
					[]string{"service_type", serviceType},
					[]string{"service_addr", serviceAddr},
					[]string{"volume", scoreService.Volume},
				},
				Value: float64(serviceInfo.Score),
				TTL:   confInterval * 2,
			})
		}
	}

	return nil
}
